﻿using IRaCIS.Application.Contracts;
using IRaCIS.Application.Interfaces;
using IRaCIS.Core.Domain.Share;
using Microsoft.AspNetCore.Mvc;
using System.Linq.Dynamic.Core;

namespace IRaCIS.Core.Application.Service
{
    [ApiExplorerSettings(GroupName = "Reviewer")]
    public class DoctorListService(IRepository<Doctor> _doctorRepository,
        IRepository<Enroll> _enrollRepository,
        IRepository<EnrollDetail> _enrollDetailRepository,
        IRepository<Trial> _trialRepository, IMapper _mapper, IUserInfo _userInfo, IStringLocalizer _localizer) : BaseService, IDoctorListQueryService
    {


        /// <summary>
        /// Reviewer列表分页查询
        /// </summary>       
        [HttpPost]
        public async Task<PageOutput<DoctorDTO>> GetDoctorSearchList(DoctorSearchDTO inQuery)
        {

            // 项目经验 多选
            var evaluationCriteriaCount = inQuery.EvaluationCriteriaIdList.Count();

            //  搜索条件 ReadingType 、Subspeciality、Title  多选
            var count = inQuery.ReadingTypeIdList.Count + inQuery.TitleIdList.Count + inQuery.SubspecialityIdList.Count;

            var guidList = inQuery.ReadingTypeIdList.Concat(inQuery.SubspecialityIdList).Concat(inQuery.TitleIdList);


            var query = _doctorRepository.AsQueryable()
                   .Where(x=>x.TrialId==null)
                   .WhereIf(inQuery.DepartmentId != null, t => t.DepartmentId == inQuery.DepartmentId)
                   .WhereIf(inQuery.SpecialityId != null, t => t.SpecialityId == inQuery.SpecialityId)
                   .WhereIf(inQuery.HospitalId != null, t => t.HospitalId == inQuery.HospitalId)
                   .WhereIf(inQuery.PositionId != null, t => t.PositionId == inQuery.PositionId)
                   .WhereIf(inQuery.RankId != null, t => t.RankId == inQuery.RankId)
                   .WhereIf(inQuery.ContractorStatus != null, t => t.CooperateStatus == inQuery.ContractorStatus)
                   .WhereIf(inQuery.InformationConfirmed != null, t => t.ResumeStatus == inQuery.InformationConfirmed)
                   .WhereIf(inQuery.AcceptingNewTrial != null, t => t.AcceptingNewTrial == inQuery.AcceptingNewTrial)
                   .WhereIf(!string.IsNullOrWhiteSpace(inQuery.Name), t => t.ChineseName.Contains(inQuery.Name) || (t.LastName + t.FirstName).Contains(inQuery.Name))
                   .WhereIf(inQuery.Nation != null, t => t.Nation == inQuery.Nation)
                   .WhereIf(evaluationCriteriaCount > 0, t => t.TrialExperienceCriteriaList.Count(t => inQuery.EvaluationCriteriaIdList.Contains(t.EvaluationCriteriaId)) == evaluationCriteriaCount)
                   ////用户类型  看到简历的范围这里需要确认  
                   //.WhereIf(_userInfo.UserTypeEnumInt == (int)UserTypeEnum.ReviewerCoordinator, t => t.UserList.Any(u => u.UserId == _userInfo.Id))
                   .WhereIf(count > 0, t => t.DoctorDicRelationList.Count(u => guidList.Contains(u.DictionaryId)) == count)
                   .WhereIf(inQuery.EnrollStatus != null && inQuery.EnrollStatus == (int)ReviewerEnrollStatus.Yes, t => t.EnrollList.Any(u => u.EnrollStatus == EnrollStatus.DoctorReading))

                   .ProjectTo<DoctorDTO>(_mapper.ConfigurationProvider);

            return await query.ToPagedListAsync(inQuery, nameof(DoctorDTO.CreateTime));


        }


        #region 入组查询相关
        /// <summary>
        /// 获取可筛选筛选及已经筛选的医生列表
        /// </summary>
        [HttpPost]
        public async Task<PageOutput<SelectionReviewerDTO>> GetSelectionReviewerList(
            ReviewerSelectionQueryDTO inQuery)
        {
            //项目配置需要的医生过滤  2表示混合
            var trialConfig = await _trialRepository.Where(s => s.Id == inQuery.TrialId).Select(t => new { t.AttendedReviewerTypeEnumList, t.TrialType }).FirstOrDefaultAsync().IfNullThrowException();

            //var nation = trialConfig.AttendedReviewerType;
            // 临床项目经验 多选
            var evaluationCriteriaCount = inQuery.EvaluationCriteriaIdList.Count();

            //  搜索条件 ReadingType 、Subspeciality、Title  多选
            var count = inQuery.ReadingTypeIdList.Count + inQuery.TitleIdList.Count + inQuery.SubspecialityIdList.Count;

            var guidList = inQuery.ReadingTypeIdList.Concat(inQuery.SubspecialityIdList).Concat(inQuery.TitleIdList);

            var query = _doctorRepository/*.WhereIf(nation != AttendedReviewerType.USAndCN,  t => t.Nation ==(int) nation)*/
             .Where(x=>x.TrialId==null)

             .WhereIf(inQuery.DepartmentId != null, t => t.DepartmentId == inQuery.DepartmentId)
             .WhereIf(inQuery.SpecialityId != null, t => t.SpecialityId == inQuery.SpecialityId)
             .WhereIf(trialConfig.TrialType == TrialType.NoneOfficial, t => t.IsVirtual == true)
             .WhereIf(trialConfig.TrialType != TrialType.NoneOfficial, t => t.IsVirtual == false)
             .WhereIf(inQuery.HospitalId != null, t => t.HospitalId == inQuery.HospitalId)
             .WhereIf(inQuery.PositionId != null, t => t.PositionId == inQuery.PositionId)
             .WhereIf(inQuery.RankId != null, t => t.RankId == inQuery.RankId)
             .WhereIf(inQuery.ContractorStatus != null, t => t.CooperateStatus == inQuery.ContractorStatus)
             .WhereIf(inQuery.InformationConfirmed != null, t => t.ResumeStatus == inQuery.InformationConfirmed)
             .WhereIf(inQuery.AcceptingNewTrial != null, t => t.AcceptingNewTrial == inQuery.AcceptingNewTrial)
             .WhereIf(!string.IsNullOrWhiteSpace(inQuery.Name), t => t.ChineseName.Contains(inQuery.Name) || (t.LastName + t.FirstName).Contains(inQuery.Name))
             .WhereIf(evaluationCriteriaCount > 0, t => t.TrialExperienceCriteriaList.Count(t => inQuery.EvaluationCriteriaIdList.Contains(t.EvaluationCriteriaId)) == evaluationCriteriaCount)
             //用户类型  看到简历的范围这里需要确认  
             //.WhereIf(_userInfo.UserTypeEnumInt == (int)UserTypeEnum.ReviewerCoordinator, t => t.UserList.Any(u => u.UserId == _userInfo.Id))
             .WhereIf(count > 0, t => t.DoctorDicRelationList.Count(u => guidList.Contains(u.DictionaryId)) == count)
             .WhereIf(inQuery.EnrollStatus != null && inQuery.EnrollStatus == (int)ReviewerEnrollStatus.Yes, t => t.EnrollList.Any(u => u.EnrollStatus == EnrollStatus.DoctorReading))

             .ProjectTo<SelectionReviewerDTO>(_mapper.ConfigurationProvider);

            var result = await query.ToPagedListAsync(inQuery);

            //是否已申请   申请时间 申请人
            var doctorStateList = await _enrollDetailRepository.Where(x => x.TrialId == inQuery.TrialId && x.EnrollStatus == EnrollStatus.HasApplyDownloadResume)
               .ProjectTo<DoctorStateModelDTO>(_mapper.ConfigurationProvider).ToListAsync();

            var doctorEnrollList =await _enrollRepository.Where(x => x.TrialId == inQuery.TrialId && x.EnrollStatus >= EnrollStatus.InviteIntoGroup)
            .ToListAsync();
            result.CurrentPageData.ToList().ForEach(doctor =>
              {
                  //简历申请列表  --处理已经申请的
                  var doctorState = doctorStateList.FirstOrDefault(t => t.DoctorId == doctor.Id && t.IntoGroupState == (int)EnrollStatus.HasApplyDownloadResume);
                  if (doctorState != null)
                  {
                      doctor.DoctorTrialState = (int)EnrollStatus.HasApplyDownloadResume;
                      doctor.IsEnroll = doctorEnrollList.Any(x => x.DoctorId == doctor.Id);
                      doctor.OptTime = doctorState.OptTime;
                      doctor.OptUserName = doctorState.OptUserName;
                  }
              });

            return result;


        }

        /// <summary>
        /// 获取提交CRO或者CRO审核的Reviewer列表
        /// </summary>
        /// <summary>
        /// 根据状态获取医生列表，入组 相关接口  （提交CRO-1） CRO确认-4  
        /// </summary>
        [HttpPost]
        public async Task<PageOutput<ConfirmationReviewerDTO>> GetSubmissionOrApprovalReviewerList(
            ReviewerSubmissionQueryDTO inQuery)
        {

            var doctorQuery = _enrollRepository.Where(x => x.TrialId == inQuery.TrialId)
                  //提交CRO  以及下载简历列表  
                  .WhereIf(inQuery.IntoGroupSearchState == 1, t => t.EnrollStatus >= EnrollStatus.HasApplyDownloadResume)
                  //CRO确认列表  状态为 已提交CRO
                  .WhereIf(inQuery.IntoGroupSearchState == 4, t => t.EnrollStatus >= EnrollStatus.HasCommittedToCRO)
                  .ProjectTo<ConfirmationReviewerDTO>(_mapper.ConfigurationProvider);

            var doctorPageList = await doctorQuery.ToPagedListAsync(inQuery);


            var enrollStateList = await _enrollDetailRepository.Where(x => x.TrialId == inQuery.TrialId)
                   //提交CRO  以及下载简历列表  
                   .WhereIf(inQuery.IntoGroupSearchState == 1, t => t.EnrollStatus == EnrollStatus.HasCommittedToCRO)
                   //CRO确认列表  状态为 已提交CRO
                   .WhereIf(inQuery.IntoGroupSearchState == 4, t => t.EnrollStatus == EnrollStatus.InviteIntoGroup)
                   .ProjectTo<DoctorStateModelDTO>(_mapper.ConfigurationProvider).ToListAsync();




            doctorPageList.CurrentPageData.ToList().ForEach(u =>
            {
                var opt = enrollStateList.FirstOrDefault(t => t.DoctorId == u.Id);
                if (opt != null)
                {
                    u.DoctorTrialState = inQuery.IntoGroupSearchState == 1 ? (int)EnrollStatus.HasCommittedToCRO : (int)EnrollStatus.InviteIntoGroup;
                    u.OptTime = opt.OptTime;
                    u.OptUserName = opt.OptUserName;
                }
                else
                {
                    u.DoctorTrialState = inQuery.IntoGroupSearchState == 1 ? (int)EnrollStatus.HasDownloadResume : (int)EnrollStatus.HasCommittedToCRO;
                }
            });

            if (_userInfo.UserTypeEnumInt == (int)UserTypeEnum.SPM)
            {
                //SPM 要看到提交的时间 提交人
                var enrollCommitList = await _enrollDetailRepository.Where(x => x.TrialId == inQuery.TrialId)
                     //提交CRO  以及下载简历列表  
                     .WhereIf(inQuery.IntoGroupSearchState == 4, t => t.EnrollStatus == EnrollStatus.HasCommittedToCRO)
                     .ProjectTo<DoctorStateModelDTO>(_mapper.ConfigurationProvider).ToListAsync();


                doctorPageList.CurrentPageData.ToList().ForEach(u =>
                {
                    var opt = enrollCommitList.FirstOrDefault(t => t.DoctorId == u.Id);
                    if (opt != null)
                    {

                        u.SubmmitTime = opt.OptTime;
                        u.SubmmitUserName = opt.OptUserName;
                    }
                });

            }

            return doctorPageList;



        }


        //public async  Task<PageOutput<ConfirmationReviewerDTO>> GetSPMSubmissionOrApprovalReviewerList(
        //    ReviewerSubmissionQueryDTO param)
        //{
        //    var doctorQuery = _enrollRepository(x => x.TrialId == param.TrialId)
        //          //提交CRO  以及下载简历列表  
        //          .WhereIf(param.IntoGroupSearchState == 1, t => t.EnrollStatus >= (int)EnrollStatus.HasApplyDownloadResume)
        //          //CRO确认列表  状态为 已提交CRO
        //          .WhereIf(param.IntoGroupSearchState == 4, t => t.EnrollStatus >= (int)EnrollStatus.HasCommittedToCRO)
        //          .ProjectTo<ConfirmationReviewerDTO>(_mapper.ConfigurationProvider);

        //    var doctorPageList = await doctorQuery.ToPagedListAsync(param.PageIndex, param.PageSize, param.SortField == "" ? "Code" : param.SortField, param.Asc);


        //    var enrollStateList = await _enrollDetailRepository.Where(x => x.TrialId == param.TrialId)
        //           //提交CRO  以及下载简历列表  
        //           .WhereIf(param.IntoGroupSearchState == 1, t => t.EnrollStatus == (int)EnrollStatus.HasCommittedToCRO)
        //           //CRO确认列表  状态为 已提交CRO
        //           .WhereIf(param.IntoGroupSearchState == 4, t => t.EnrollStatus == (int)EnrollStatus.InviteIntoGroup)
        //           .ProjectTo<DoctorStateModelDTO>(_mapper.ConfigurationProvider).ToListAsync();


        //    doctorPageList.CurrentPageData.ToList().ForEach(u =>
        //    {
        //        var opt = enrollStateList.FirstOrDefault(t => t.DoctorId == u.Id);
        //        if (opt != null)
        //        {
        //            u.DoctorTrialState = param.IntoGroupSearchState == 1 ? (int)EnrollStatus.HasCommittedToCRO : (int)EnrollStatus.InviteIntoGroup;
        //            u.OptTime = opt.OptTime;
        //            u.OptUserName = opt.OptUserName;
        //        }
        //    });

        //    return doctorPageList;
        //}

        /// <summary>
        /// 获取项目下医生入组状态列表[Confirmation]
        /// </summary>
        [HttpPost]
        public async Task<PageOutput<ConfirmationReviewerDTO>> GetConfirmationReviewerList(
            ReviewerConfirmationQueryDTO inQuery)
        {


            #region 连表查询
            //var query = from enroll in _enrollRepository.Where(x => x.TrialId == inQuery.TrialId && x.EnrollStatus >= EnrollStatus.InviteIntoGroup)
            //            join enrollDetail in _enrollDetailRepository.Where(x => x.TrialId == inQuery.TrialId && x.EnrollStatus >= EnrollStatus.InviteIntoGroup) on new { enroll.DoctorId, enroll.EnrollStatus } equals new { enrollDetail.DoctorId, enrollDetail.EnrollStatus } into enrollDetailtemp
            //            from leftenrollDetail in enrollDetailtemp.DefaultIfEmpty()
            //            select new ConfirmationReviewerDTO()
            //            {
            //                DoctorTrialState = (int)leftenrollDetail.EnrollStatus,
            //                OptTime = leftenrollDetail.CreateTime,
            //                UserTypeEnum = leftenrollDetail.CreateUserRole.UserTypeEnum,
            //                OptUserName = leftenrollDetail.CreateUserRole.UserName,
            //                SubmmitTime = leftenrollDetail.CreateTime,
            //                SubmmitUserName = leftenrollDetail.CreateUserRole.UserName,
            //                //BlindName= enroll.BlindName,
            //                //BlindNameCN= enroll.BlindNameCN,
            //                SpecialityId = enroll.Doctor.SpecialityId,
            //                SpecialityCN = enroll.Doctor.Speciality.ValueCN,
            //                Speciality = enroll.Doctor.Speciality.Value,
            //                //SpecialityOther=enroll.Doctor.Speciality.SpecialityOther,
            //                //SpecialityOtherCN= enroll.Doctor.Speciality.SpecialityOtherCN,
            //                DictionaryList = enroll.Doctor.DoctorDicRelationList.Where(t => t.KeyName == StaticData.ReadingType || t.KeyName == StaticData.Subspeciality).Select(t => t.Dictionary).OrderBy(t => t.ShowOrder).Select(x => new DicView()
            //                {
            //                    Id = x.Id,
            //                    ParentCode = x.Parent.Code,
            //                    ShowOrder = x.ShowOrder,
            //                    Value = x.Value,
            //                    ValueCN = x.ValueCN
            //                }).ToList(),

            //                //HospitalId= enroll.BlindName

            //                Id = enroll.Doctor.Id,
            //                Code = enroll.DoctorUser.UserName,
            //                //FirstName=enroll.FirstName,
            //                //LastName = enroll.LastName,
            //                //ChineseName = enroll.ChineseName,
            //                //City = enroll.City,
            //                //HospitalName = enroll.HospitalName,
            //                //Country = enroll.Country,
            //            };


            //var result = await query
            //     .WhereIf(inQuery.OptStartTime != null, x => x.OptTime >= inQuery.OptStartTime)
            //     .WhereIf(inQuery.OptEndTime != null, x => x.OptTime <= inQuery.OptEndTime)
            //       .WhereIf(inQuery.UserTypeEnum != null, x => x.UserTypeEnum == inQuery.UserTypeEnum)
            //         .WhereIf(inQuery.Code.IsNotNullOrEmpty(), x => x.Code.Contains(inQuery.Code))

            //        .WhereIf(inQuery.OptUserName.IsNotNullOrEmpty(), x => x.OptUserName.Contains(inQuery.OptUserName))
            //       .ToPagedListAsync(inQuery);
            #endregion


            #region 查询第一个

            // 这里这样写的原因是因为 要取最新的入组详情  然后还要根据时间筛选  入组信息
            var query = _enrollDetailRepository
                .Where(x => x.TrialId == inQuery.TrialId && x.EnrollStatus >= EnrollStatus.InviteIntoGroup);
            var enrollStateList = await query
                 .ProjectTo<DoctorStateModelDTO>(_mapper.ConfigurationProvider).ToListAsync();
            enrollStateList = enrollStateList.GroupBy(e => e.DoctorId)
               .Select(g => g.OrderByDescending(e => e.OptTime).FirstOrDefault()).ToList();

            enrollStateList = enrollStateList
               .WhereIf(inQuery.OptStartTime != null, x => x.OptTime >= inQuery.OptStartTime)
              .WhereIf(inQuery.OptEndTime != null, x => x.OptTime <= inQuery.OptEndTime)
              .WhereIf(inQuery.UserTypeEnum != null, x => x.UserTypeEnum == inQuery.UserTypeEnum)
              .ToList();
            #endregion





            var doctorids =  enrollStateList.Select(x => x.DoctorId).ToList();


            var doctorQuery = _enrollRepository.Where(x => x.TrialId == inQuery.TrialId && x.EnrollStatus >= EnrollStatus.InviteIntoGroup)





           .WhereIf(inQuery.OptStartTime != null || inQuery.OptEndTime != null || inQuery.UserTypeEnum != null, x => doctorids.Contains(x.DoctorId))
            .ProjectTo<ConfirmationReviewerDTO>(_mapper.ConfigurationProvider)
            .WhereIf(inQuery.Code.IsNotNullOrEmpty(), x => x.Code.Contains(inQuery.Code))
           
            .WhereIf(inQuery.OptUserName.IsNotNullOrEmpty(), x => x.OptUserName.Contains(inQuery.OptUserName));

            var doctorPageList = await doctorQuery.ToPagedListAsync(inQuery);


            doctorPageList.CurrentPageData.ToList().ForEach(u =>
            {
                u.DoctorTrialState = (int)EnrollStatus.InviteIntoGroup;
                var opt = enrollStateList.OrderByDescending(x=>x.OptTime).FirstOrDefault(t => t.DoctorId == u.Id);
                if (opt != null)
                {
                    u.DoctorTrialState = opt.IntoGroupState;
                    u.OptTime = opt.OptTime;
                    u.UserTypeEnum = opt.UserTypeEnum;
                    u.OptUserName = opt.OptUserName;
                }
            });

            return doctorPageList;

        }



        /// <summary>
        /// 获取项目下医生列表
        /// </summary>
        /// <param name="inQuery"></param>
        /// <returns></returns>
        [HttpPost]
        public async Task<PageOutput<TrialDoctorInfo>> GetTrialDoctorList(
            GetTrialDoctorListInDto inQuery)
        {


            var doctorQuery = _enrollRepository.Where(x => x.TrialId == inQuery.TrialId && x.EnrollStatus == EnrollStatus.ConfirmIntoGroup)
                
               .ProjectTo<TrialDoctorInfo>(_mapper.ConfigurationProvider);

            var doctorPageList = await doctorQuery
                .WhereIf(inQuery.HospitalId!=null,x=>x.HospitalId==inQuery.HospitalId.Value)
                .WhereIf(inQuery.SpecialityId != null, x => x.SpecialityId == inQuery.SpecialityId.Value)
                .WhereIf(inQuery.SubspecialityId != null, x => x.DictionaryList.Any(x=>x.Id==inQuery.SubspecialityId.Value))
                .ToPagedListAsync(inQuery);

            var enrollStateList = await _enrollDetailRepository.Where(x => x.TrialId == inQuery.TrialId && x.EnrollStatus > EnrollStatus.InviteIntoGroup)
                 .ProjectTo<DoctorStateModelDTO>(_mapper.ConfigurationProvider).ToListAsync();



            doctorPageList.CurrentPageData.ToList().ForEach(u =>
            {
                u.DoctorTrialState = (int)EnrollStatus.InviteIntoGroup;
                var opt = enrollStateList.FirstOrDefault(t => t.DoctorId == u.Id);
                if (opt != null)
                {
                    u.DoctorTrialState = opt.IntoGroupState;
                    u.OptTime = opt.OptTime;
                    u.OptUserName = opt.OptUserName;
                }
            });

            return doctorPageList;

        }



        #endregion




    }
}

